﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using MandelbrotLib;

namespace MandelbrotWin
{
    public partial class VnitrniOkno : Form
    {
        private string nazevSouboru = "nepojmenovaný";
        private bool ulozeno = true;
        private bool pojmenovano = false;
        private int sirka = MandelbrotSet.WIDTH;
        private int vyska = MandelbrotSet.HEIGHT;
        private double realna = MandelbrotSet.WIDTHX;
        private double imaginarni = MandelbrotSet.WIDTHY;
        private double meritko = MandelbrotSet.SCALE;
        private ComplexNumber pos = new ComplexNumber(MandelbrotSet.POSR,
            MandelbrotSet.POSI);
        private int exponent = MandelbrotSet.EXPONENT;
        private int iterace = MandelbrotSet.MAXITER;
        private string[] spec = new string[] { MandelbrotSet.SPECCOL,
            MandelbrotSet.SPECCOL, MandelbrotSet.SPECCOL };
        //private Palette paleta = new Palette("", true);
        private string souborPalety = "";
        private bool special = !MandelbrotSet.OWNPALETTE;

        public string NazevSouboru
        {
            get { return this.nazevSouboru; }
            set { this.nazevSouboru = value; }
        }

        public bool Ulozeno
        {
            get { return this.ulozeno; }
            set { this.ulozeno = value; }
        }

        public bool Pojmenovano
        {
            get { return this.pojmenovano; }
            set { this.pojmenovano = value; }
        }

        public int Sirka
        {
            get { return this.sirka; }
            set { this.sirka = (value > 0) ? value : this.sirka; }
        }

        public int Vyska
        {
            get { return this.vyska; }
            set { this.vyska = (value > 0) ? value : this.vyska; }
        }

        public double Realna
        {
            get { return this.realna; }
            set { this.realna = ((value > 0.0) && (value <= 2.0)) ? value : this.realna; }
        }

        public double Imaginarni
        {
            get { return this.imaginarni; }
            set { this.imaginarni = ((value > 0.0) && (value <= 2.0)) ?
                  value : this.imaginarni; }
        }

        public double Meritko
        {
            get { return this.meritko; }
            set { this.meritko = ((value > 0.0) && (value <= 1.0)) ?
                  value : this.meritko; }
        }

        public ComplexNumber Pos
        {
            get { return this.pos; }
            set
            {
                if ((value.R >= -this.Realna) && (value.R <= this.Realna) &&
                    (value.I >= -this.Imaginarni) && (value.I <= this.Imaginarni))
                {
                    this.pos = value;
                }
            }
        }

        public int Exponent
        {
            get { return this.exponent; }
            set { this.exponent = (value >= 2) ? value : this.exponent; }
        }

        public int Iterace
        {
            get { return this.iterace; }
            set { this.iterace = ((value >= 0) && (value <= 255)) ? value : this.iterace; }
        }

        public string SouborPalety
        {
            get { return this.souborPalety; }
            set { this.souborPalety = value; }
        }

        public bool Special
        {
            get { return this.special; }
            set { this.special = value; }
        }

        public int PalR
        {
            get
            {
                switch (this.spec[0])
                {
                    case "+":
                        return -1; // Tady break být doopravdy nemusí...
                    case "-":
                        return -2; // Protože Unreachable code detected :)
                    default:
                        return int.Parse(this.spec[0]);
                }
            }
            set
            {
                switch (value)
                {
                    case -1:
                        this.spec[0] = "+";
                        break;
                    case -2:
                        this.spec[0] = "-";
                        break;
                    default:
                        this.spec[0] = value.ToString();
                        break;
                }
            }
        }

        public int PalG
        {
            get
            {
                switch (this.spec[1])
                {
                    case "+":
                        return -1; // Tady break být doopravdy nemusí...
                    case "-":
                        return -2; // Protože Unreachable code detected :)
                    default:
                        return int.Parse(this.spec[1]);
                }
            }
            set
            {
                switch (value)
                {
                    case -1:
                        this.spec[1] = "+";
                        break;
                    case -2:
                        this.spec[1] = "-";
                        break;
                    default:
                        this.spec[1] = value.ToString();
                        break;
                }
            }
        }

        public int PalB
        {
            get
            {
                switch (this.spec[2])
                {
                    case "+":
                        return -1; // Tady break být doopravdy nemusí...
                    case "-":
                        return -2; // Protože Unreachable code detected :)
                    default:
                        return int.Parse(this.spec[2]);
                }
            }
            set
            {
                switch (value)
                {
                    case -1:
                        this.spec[2] = "+";
                        break;
                    case -2:
                        this.spec[2] = "-";
                        break;
                    default:
                        this.spec[2] = value.ToString();
                        break;
                }
            }
        }

        public Bitmap BMP
        {
            get { return (Bitmap)this.pictureBox1.Image; }
        }

        public VnitrniOkno()
        {
            InitializeComponent();
            this.Text = nazevSouboru;
            this.UpravRozmery();
        }

        public void UpravRozmery()
        {
            this.pictureBox1.Image = (Image)(new Bitmap(this.Sirka, this.Vyska,
                System.Drawing.Imaging.PixelFormat.Format24bppRgb));
            this.pictureBox1.Width = this.Sirka;
            this.pictureBox1.Height = this.Vyska;
            this.Ulozeno = false;
        }

        private void vykresleno(object sender, Bitmap bmp)
        {
            this.nastavObrazek(bmp);
        }

        delegate void nastavObrazekCallback(Bitmap bmp);

        // Tato metoda tu je proto, že není bezpečené upravovat vlastnosti prvků
        // okna nakříž mezi vlákny.
        private void nastavObrazek(Bitmap bmp)
        {
            if (this.pictureBox1.InvokeRequired)
            {
                nastavObrazekCallback n = new nastavObrazekCallback(this.nastavObrazek);
                this.Invoke(n, new object[] { bmp });
            }
            else
            {
                this.pictureBox1.Image = (Image)bmp;
                this.pictureBox1.Refresh();
                this.Ulozeno = false;
                this.Text = this.NazevSouboru;
            }
        }

        public void ZacniVykreslovat()
        {
            this.Text = "Vykreslování množiny...";
            Palette paleta;
            if (!this.Special)
            {
                paleta = new Palette(this.SouborPalety, this.Special);
                paleta.LoadFromFile();
            }
            else
            {
                paleta = new Palette("", this.Special);
            }
            paleta.SetSpecial(this.spec[0], this.spec[1], this.spec[2]);
            MandelbrotSet m = new MandelbrotSet(this.Sirka, this.Vyska, this.Realna,
                this.Imaginarni, this.Meritko, this.Pos, this.Exponent, this.Iterace,
                paleta);
            m.OnBitmapDrawed += this.vykresleno;
            m.BeginDraw();
        }

        public void Uloz(bool ulozitJako)
        {
            if (ulozitJako)
            {
                if (this.saveFileDialog1.ShowDialog() ==
                    System.Windows.Forms.DialogResult.OK)
                {
                    this.NazevSouboru = this.saveFileDialog1.FileName;
                }
                else
                {
                    return;
                }
            }
            try
            {
                this.BMP.Save(this.NazevSouboru, System.Drawing.Imaging.ImageFormat.Png);
                this.Ulozeno = true;
                this.Pojmenovano = true;
                this.Text = this.NazevSouboru;
            }
            catch (Exception e)
            {
                MessageBox.Show(string.Format("Při ukládání obrázku došlo k chybě:"
                    + "{0}", e.Message), "Chyba", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            
        }

        private void kontrola(object sender, FormClosingEventArgs e)
        {
            if (!this.Ulozeno)
            {
                if (MessageBox.Show("Obrázek není uložený. Přejete si jej uložit?", "",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Question) ==
                    System.Windows.Forms.DialogResult.Yes)
                {
                    this.Uloz(!this.Pojmenovano);
                }
            }
        }
    }
}
